/*
  IR_Hacks_UNI_Remote
  -------------------
  This sketch demonstrates a universal remote control using the amomii IR Hacks shield. 
  It allows you to switch between four different remotes and control various devices 
  using an IR transmitter. The OLED display shows the selected remote name. 
  You can also view temperature and humidity readings by pressing a button.

  Note: This code serves as a template. Users must enter the required information 
  for their specific remotes. This will involve finding the IR signals for their remotes 
  using the IR Hacks modules and example code from the IRremote library. Detailed instructions 
  are provided in the IR Hacks Getting Started guide, available in the downloads area 
  at the IR Hacks section at amomii.com/pages/downloads.

  Libraries used:
  - IRremote for IR communication
  - lcdgfx for OLED display control
  - DHT for temperature and humidity sensor

  Connections:
  - IR send pin: Digital pin 3
  - Buzzer pin: Digital pin 12
  - DHT11 sensor pin: Analog pin A0
  - Keypad connected to pins 4, 5, 6, 7 (rows) and 8, 9, 10, 11 (columns)
  - OLED display connected via I2C
  - SW1 button connected to Analog pin A1
  
  Created by Shaun Armstrong, 2024
*/


#include <IRremote.hpp>  // IR Communication
#include <Wire.h>        // I2C
#include <lcdgfx.h>
#include "lcdgfx_gui.h"
#include "DHT.h"
#include "images.h"

/////////////////////////////////// OLED //////////////////////////////
DisplaySSD1306_128x64_I2C display(-1);

int sw1 = A1;
bool sw1_long = false;

//////////////////////////////////// Keypad /////////////////////////

int rows[] = { 7, 6, 5, 4 };
int columns[] = { 11, 10, 9, 8 };

const int ROWS = 4;
const int COLS = 4;

char keys[ROWS][COLS] = {
  { '1', '2', '3', 'A' },
  { '4', '5', '6', 'B' },
  { '7', '8', '9', 'C' },
  { '*', '0', '#', 'D' }
};

// Define remote names
const char* remoteNames[4] = { "Remote A", "Remote B", "Remote C", "Remote D" };
int currentRemote = 0;  // Index for the currently selected remote
int lastRemote = -1;    // To keep track of the previous remote selection

// Define the pin to send IR signals
const int IR_SEND_PIN = 3;  // Replace with your actual send pin

// Define buzzer pin
int buz = 12;

//////////////////////DHT11///////////////////
#define DHTPIN A0
#define DHTTYPE DHT11  // DHT 11
DHT dht(DHTPIN, DHTTYPE);

void setup() {
  Serial.begin(115200);

  // Initialize the IR sender
  IrSender.begin(IR_SEND_PIN);

  // Initialize the OLED display
  display.setFixedFont(ssd1306xled_font6x8);
  display.begin();
  display.getInterface().flipHorizontal(1);
  display.getInterface().flipVertical(1);
  display.fill(0x00);
  display.drawBitmap1(0, 0, 128, 64, ir_logo);


  // Initialize the keyboard
  for (int i = 0; i < COLS; i++) {
    pinMode(columns[i], INPUT_PULLUP);
  }
  for (int i = 0; i < ROWS; i++) {
    pinMode(rows[i], OUTPUT);
    digitalWrite(rows[i], HIGH);
  }

  pinMode(sw1, INPUT_PULLUP);

  // Display the initial remote name
  displayRemoteName();

  //Buzzer pinMode
  pinMode(buz, OUTPUT);

  //DHT11
  dht.begin();
}

void selectFX() {
  for (int i = 500; i < 550; i++) {
    tone(buz, i * 2);
    delay(1);
  }
  noTone(buz);
}

void displayRemoteName() {
  display.clear();

  display.drawBitmap1(0, 0, 128, 64, ir_logo);
  display.printFixed(0, 55, remoteNames[currentRemote]);
}

void checkKeypad() {
  bool flag = false;
  int pressedRow = -1;
  int pressedCol = -1;

  for (int i = 0; i < ROWS; i++) {
    for (int x = 0; x < ROWS; x++) {
      digitalWrite(rows[x], HIGH);
    }
    digitalWrite(rows[i], LOW);

    for (int j = 0; j < COLS; j++) {
      if (digitalRead(columns[j]) == LOW) {
        pressedRow = i;
        pressedCol = j;
        flag = true;
        break;
      }
    }

    if (flag) break;
  }

  if (flag) {
    char key = keys[pressedRow][pressedCol];
    handleKeyPress(key);
    delay(300);
  }
}

void handleKeyPress(char key) {
  switch (key) {
    case 'A':
      currentRemote = 0;
      break;
    case 'B':
      currentRemote = 1;
      break;
    case 'C':
      currentRemote = 2;
      break;
    case 'D':
      currentRemote = 3;
      break;
    default:
      executeCommand(key);
      break;
  }

  // Update the display only if the remote selection has changed
  if (currentRemote != lastRemote) {
    selectFX();
    displayRemoteName();
    lastRemote = currentRemote;
  }
}

void executeCommand(char key) {
  switch (currentRemote) {
    case 0:
      remote_1(key);
      break;
    case 1:
      remote_2(key);
      break;
    case 2:
      remote_3(key);
      break;
    case 3:
      remote_4(key);
      break;
  }
}

void remote_1(char key) {

  switch (key) {
    case '0':
      // Paste the command for key '0' here
      break;
    case '1':
      // Paste the command for key '1' here
      break;
    case '2':
      // Paste the command for key '2' here
      break;
    case '3':
      // Paste the command for key '3' here
      break;
    case '4':
      // Paste the command for key '4' here
      break;
    case '5':
      // Paste the command for key '5' here
      break;
    case '6':
      // Paste the command for key '6' here
      break;
    case '7':
      // Paste the command for key '7' here
      break;
    case '8':
      // Paste the command for key '8' here
      break;
    case '9':
      // Paste the command for key '9' here
      break;
    case '*':
      // Paste the command for key '*' here
      break;
    case '#':
      // Paste the command for key '#' here
      break;
  }
}

void remote_2(char key) {

  switch (key) {
    case '0':
      // Paste the command for key '0' here
      break;
    case '1':
      // Paste the command for key '1' here
      break;
    case '2':
      // Paste the command for key '2' here
      break;
    case '3':
      // Paste the command for key '3' here
      break;
    case '4':
      // Paste the command for key '4' here
      break;
    case '5':
      // Paste the command for key '5' here
      break;
    case '6':
      // Paste the command for key '6' here
      break;
    case '7':
      // Paste the command for key '7' here
      break;
    case '8':
      // Paste the command for key '8' here
      break;
    case '9':
      // Paste the command for key '9' here
      break;
    case '*':
      // Paste the command for key '*' here
      break;
    case '#':
      // Paste the command for key '#' here
      break;
  }
}

void remote_3(char key) {
  switch (key) {
    case '0':
      // Paste the command for key '0' here
      break;
    case '1':
      // Paste the command for key '1' here
      break;
    case '2':
      // Paste the command for key '2' here
      break;
    case '3':
      // Paste the command for key '3' here
      break;
    case '4':
      // Paste the command for key '4' here
      break;
    case '5':
      // Paste the command for key '5' here
      break;
    case '6':
      // Paste the command for key '6' here
      break;
    case '7':
      // Paste the command for key '7' here
      break;
    case '8':
      // Paste the command for key '8' here
      break;
    case '9':
      // Paste the command for key '9' here
      break;
    case '*':
      // Paste the command for key '*' here
      break;
    case '#':
      // Paste the command for key '#' here
      break;
  }
}

void remote_4(char key) {
  switch (key) {
    case '0':
      // Paste the command for key '0' here
      break;
    case '1':
      // Paste the command for key '1' here
      break;
    case '2':
      // Paste the command for key '2' here
      break;
    case '3':
      // Paste the command for key '3' here
      break;
    case '4':
      // Paste the command for key '4' here
      break;
    case '5':
      // Paste the command for key '5' here
      break;
    case '6':
      // Paste the command for key '6' here
      break;
    case '7':
      // Paste the command for key '7' here
      break;
    case '8':
      // Paste the command for key '8' here
      break;
    case '9':
      // Paste the command for key '9' here
      break;
    case '*':
      // Paste the command for key '*' here
      break;
    case '#':
      // Paste the command for key '#' here
      break;
  }
}

void dht11() {
  float t = dht.readTemperature();
  float h = dht.readHumidity();

  display.clear();
  display.printFixed(0, 20, "Temperature: ");
  display.printFixed(78, 20, String(t).c_str());
  display.printFixed(110, 20, "C");

  display.printFixed(0, 40, "Humidity: ");
  display.printFixed(78, 40, String(h).c_str());
  display.printFixed(110, 40, "%");
}



void loop() {
  checkKeypad();
  
  if (!digitalRead(sw1) && sw1_long == false) {
    selectFX();
    sw1_long = true;
    dht11();
  }
  if (digitalRead(sw1)){
    sw1_long = false;
  }
}
