/*
  TEST 2 for the amomii IR Shield. Written by Shaun Armstrong on July 7, 2024.

  This code tests the IR transmitter and receiver.

  It transmits a signal and tests to see if the receiver detects it.
  It should work fine without, but reflecting the IR signals back at the device 
  with a mirror may improve reliability.

  Upload this code to your UNO with the IR Hacks shield attached. 
  After the upload is complete, open the serial monitor and follow the instructions.
  Note: you must set the Baud Rate 115200
*/

#include "PinDefinitionsAndMore.h"  // Define macros for input and output pin etc.
#include <IRremote.hpp>             // IR Communication
#include "images.h"
#include <lcdgfx.h>
#include "lcdgfx_gui.h"

#define SW1 A1
#define SW2 A2

DisplaySSD1306_128x64_I2C display(-1);


void setup() {
  Serial.begin(115200);

  pinMode(SW1, INPUT_PULLUP);
  pinMode(SW2, INPUT_PULLUP);

  IrSender.begin();           // Initialize the IR sender
  IrReceiver.begin(2, true);  // Initialize the IR receiver on pin 2, with debugging output enabled

  //////////////////////////////////// blink ////////////////////
  display.begin();
  display.setFixedFont(ssd1306xled_font6x8);


  display.getInterface().flipHorizontal(1);
  display.getInterface().flipVertical(1);

  display.clear();

  display.fill(0x00);
  display.drawBitmap1(0, 0, 128, 64, ir_logo);

}

void sendIRCode(unsigned long code) {
  unsigned int address = code & 0xFFFF;
  unsigned char command = (code >> 16) & 0xFF;
  IrSender.sendNEC(address, command, 0);
}

bool checkIRCode(unsigned long expectedCode) {
  if (IrReceiver.decode()) {
    unsigned long receivedCode = IrReceiver.decodedIRData.decodedRawData;
    Serial.print("Received IR code: ");
    Serial.println(receivedCode, HEX);
    IrReceiver.resume();
    return receivedCode == expectedCode;
  }
  return false;
}

bool testIR(unsigned long code, int buttonPin, unsigned long expectedCode) {
  Serial.print("Press Blink Button ");
  Serial.print(buttonPin == SW1 ? "SW1" : "SW2");
  Serial.println(" to transmit your IR signal");

  while (true) {
    if (!digitalRead(buttonPin)) {
      Serial.println("Press detected");
      sendIRCode(code);
      delay(200);
      Serial.println("Waiting for IR signal...");

      unsigned long startTime = millis();
      bool received = false;

      while (millis() - startTime < 1000) {  // Wait for up to 1 second
        if (checkIRCode(expectedCode)) {
          Serial.println("RECEIVED: PASS");
          received = true;
          break;
        }
      }

      if (!received) {
        Serial.println("NOT RECEIVED: FAIL");
      }
      return received;
    }
  }
}

void loop() {
  Serial.println(F("Time to test the IR Transmitter and Receiver"));
  bool sw1Result = testIR(0xF30CFF00, SW1, 0xF30CFF00);  // Correct expected code for SW1
  bool sw2Result = testIR(0xE916FF00, SW2, 0xE916FF00);  // Correct expected code for SW2

  if (sw1Result && sw2Result) {
    Serial.println(F("Both tests passed. Your IR receiver and transmitter appear to be working properly."));
  } else {
    Serial.println(F("One or both tests failed. Please check your soldering and connections and try again."));
  }

  while (true)
    ;
}
